/*
 * File: MyGame.js 
 * This is the logic of our game. For now, this is very simple.
 */
/*jslint node: true, vars: true */
/*global gEngine, SimpleShader, Transform  */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";  // Operate in Strict mode such that variables must be declared before used!

function ClassExample() { this.mMesh = null; }

ClassExample.prototype.createCustomMesh = function (meshInString) {
    this.mMesh = new OBJ.Mesh(meshInString);   // 1. Creates a Mesh Object
    OBJ.initMeshBuffers(gEngine.Core.getGL(), 
            this.mMesh);                       // 2. Initialize WebGL Buffers
    this.mConstColorShader = new SimpleShader( // 
        "src/GLSLShaders/SimpleVS.glsl",       // 3. Shader for the Mesh
        "src/GLSLShaders/SimpleFS.glsl");      // 
    this.mXf = new Transform();                // 4. Transform Object for the Mesh
    this.mXf.setXPos(200);                     // 5. Sets the X-size of the object
    this.mXf.setYPos(200);                     // 6. Sets the y-size of the object
    this.mXf.setSize(30, 30);                  // 7. Sets the size of the mesh
};

ClassExample.prototype.draw = function (camera) {
    gEngine.Core.clearCanvas([0.9, 0.9, 0.9, 1]); // 8. Clear canvas
    camera.setupViewProjection();                 // 9. Camera Magic

    if (this.mMesh !== null) {
        var gl = gEngine.Core.getGL();            //10. Gets a reference to WebGL
        this.mConstColorShader.activateShader(    //11. Activates the shader
            this.mMesh.vertexBuffer,
            [1, 0, 0, 1],
            camera.getVPMatrix());
        this.mConstColorShader.loadObjectTransform(this.mXf.getXform());
                                                 //12. sets the mesh transform

        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,   //13. Next two lines draws
                this.mMesh.indexBuffer);
        gl.drawElements(gl.TRIANGLES,
                this.mMesh.indexBuffer.numItems, gl.UNSIGNED_SHORT, 0);
    }
};
