/* 
 * File: MainController.js
 * Container controller for the entire world
 */

/*jslint node: true, vars: true, bitwise: true */
/*global angular, document, ClassExample, Camera, CanvasMouseSupport, FileReader */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";

// Creates the "backend" logical support for appMyExample
var myModule = angular.module("appMyExample", ["CSS450Timer", "CSS450Slider", "CSS450Xform"]);

// registers the constructor for the controller
// NOTE: the constructor is only called _AFTER_ the </body> tag is encountered
//       this code does NOT run until the end of loading the HTML page
myModule.controller("MainCtrl", function ($scope) {
    // Initialize the graphics system
    gEngine.Core.initializeWebGL('GLCanvas');

       // this is the model
    $scope.mMyWorld = new ClassExample();
    $scope.mView = new Camera();
    $scope.mCanvasMouse = new CanvasMouseSupport('GLCanvas');
    $scope.mSelectedXform = new Transform();
    $scope.mForceRedraw = false;

    $scope.mainTimerHandler = function () {
        // $scope.mMyWorld.update();
        $scope.mMyWorld.draw($scope.mView);
    };

    $scope.moveHandler = function (event) {
        if (event.which === 1) { // move with LMB
            var x = $scope.mCanvasMouse.getPixelXPos(event);
            var y = $scope.mCanvasMouse.getPixelYPos(event);
            if ($scope.mSelectedXform !== null) {
                $scope.mSelectedXform.setXPos(x);
                $scope.mSelectedXform.setYPos(y);
                $scope.mForceRedraw = true;
            }
        }
    };

    $scope.acceptFiles = function (event) {
        var input = event.target;
        var reader = new FileReader();
        reader.onload = function () {
            var xf = $scope.mMyWorld.createCustomMesh(reader.result);
            $scope.mSelectedXform = xf;
        };
        reader.readAsText(input.files[0]);
    };

});