/* 
 * File: MainController.js
 * Container controller for the entire world
 */

/*jslint node: true, vars: true, bitwise: true */
/*global angular, document, ClassExample, Camera, CanvasMouseSupport */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";

// Creates the "backend" logical support for appMyExample
var myModule = angular.module("appMyExample", ["CSS450Timer", "CSS450Slider", "CSS450Xform"]);

// registers the constructor for the controller
// NOTE: the constructor is only called _AFTER_ the </body> tag is encountered
//       this code does NOT run until the end of loading the HTML page
myModule.controller("MainCtrl", function ($scope) {
    // Initialize the graphics system
    gEngine.Core.initializeWebGL('GLCanvas');
    $scope.mCanvasMouse = new CanvasMouseSupport('GLCanvas');
    
    // All the mouse coordinate points
    $scope.mClientX = 0;
    $scope.mClientY = 0;
    $scope.mCanvasX = 0;
    $scope.mCanvasY = 0;
    

       // this is the model
    $scope.mMyWorld = new ClassExample();

    $scope.mView = new Camera(
                [400, 300],         // wc Center
                800,                // wc Wdith
                [0, 0, 800, 600]);  // viewport: left, bottom, width, height
            
    // small view support
    $scope.setSmallViewWC = function () {
        $scope.mSmallView.setWCWidth(parseInt($scope.mSmallViewWCWidth));
    };
    $scope.setSmallViewWCCenter = function () {
        $scope.mSmallView.setWCCenter(
            parseInt($scope.mSmallViewWCCenter[0]),
            parseInt($scope.mSmallViewWCCenter[1])
        );
    };
    $scope.setSmallViewport = function () {
        var v = $scope.mSmallView.getViewport();
        var i;
        for (i=0; i<4; i++)
            v[i] = parseInt($scope.mSmallViewport[i]);
    };
    $scope.mSmallViewWCWidth = 200;
    $scope.mSmallViewport = [100, 200, 200, 200];
    $scope.mSmallViewWCCenter = [500, 400];
    $scope.mSmallView = new Camera(
                [0, 0],// wc Center
                10, // wc width
                [0, 0, 100, 100]);    // viewport: left, bottom, width, height
    $scope.mSmallView.setBackgroundColor([0.9, 0.7, 0.7, 1]);
    $scope.setSmallViewWC();
    $scope.setSmallViewWCCenter();
    $scope.setSmallViewport();

    $scope.mainTimerHandler = function () {
        // Step E: Clear the canvas
        gEngine.Core.clearCanvas([0.9, 0.9, 0.9, 1]);        // Clear the canvas
        //
        // $scope.mMyWorld.update();
        $scope.mMyWorld.draw($scope.mView);

        $scope.mMyWorld.draw($scope.mSmallView);
    };

    // Mouse support for creation of objects
    $scope.defineSquare = function (event) {
        $scope.mMyWorld.defineCenter(
            $scope.mCanvasMouse.getPixelXPos(event),
            $scope.mCanvasMouse.getPixelYPos(event));
        $scope.mSelectedXform = $scope.mMyWorld.currentObject().getXform();
    };

    $scope.dragSquare = function (event) {
        $scope.mClientX = event.clientX;
        $scope.mClientY = event.clientY;
        $scope.mCanvasX = $scope.mCanvasMouse.getPixelXPos(event);
        $scope.mCanvasY = $scope.mCanvasMouse.getPixelYPos(event);
        
        // console.log("dragging");
        switch (event.which) {
        case 1: // left
            $scope.mMyWorld.defineWidth($scope.mCanvasX, $scope.mCanvasY);
                
            break;
        }
    };

});